// LoadProcWrapperBase.cpp: implementation of the LoadProcWrapperBase class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "LoadProcWrapperBase.h"




//////////////////////////////////////////////////////////////////////
// LoadProcWrapperBase
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

LoadProcWrapperBase::LoadProcWrapperBase()
 : m_szDllName(NULL), 
   m_szFullPath(NULL), 
   m_szExportName(NULL), 
   m_hDll(NULL), 
   m_pf(NULL),
   m_bThrowOnMissing(false)
{
   for ( int idx = 0; idx < sizeof(m_szDllPath) / sizeof(m_szDllPath[0]); idx++ ) {
      m_szDllPath[idx] = NULL;
   }
} // LoadProcWrapperBase::LoadProcWrapperBase()

LoadProcWrapperBase::~LoadProcWrapperBase() 
{
   Reset();
   ResetPaths();
   delete[] m_szDllName;
   delete[] m_szExportName;
} // LoadProcWrapperBase::~LoadProcWrapperBase() 

void LoadProcWrapperBase::Reset(void)
{
   if ( m_hDll != NULL ) {
      FreeLibrary(m_hDll);
      m_hDll = NULL;
   }
   m_pf = NULL;
} // LoadProcWrapperBase::Reset()

void LoadProcWrapperBase::SetPreferredPath(IN const TCHAR * const _p1)
{
   ResetPaths();
   SetPreferredPathI(_p1, 0);
} // (1) LoadProcWrapperBase::SetPreferredPath()

void LoadProcWrapperBase::SetPreferredPath(IN const TCHAR * const _p1, IN const TCHAR * const _p2)
{
   ResetPaths();
   SetPreferredPathI(_p1, 0);
   SetPreferredPathI(_p2, 1);
} // (2) LoadProcWrapperBase::SetPreferredPath()

void LoadProcWrapperBase::SetPreferredPath(IN const TCHAR * const _p1, IN const TCHAR * const _p2, IN const TCHAR * const _p3)
{
   ResetPaths();
   SetPreferredPathI(_p1, 0);
   SetPreferredPathI(_p2, 1);
   SetPreferredPathI(_p3, 2);
} // (3) LoadProcWrapperBase::SetPreferredPath()

const TCHAR * const LoadProcWrapperBase::GetModuleLoadPath(void)
{
   if ( m_szFullPath == NULL ) {
      if ( m_hDll != NULL ) {
         DWORD    dwSize;
         TCHAR    szDllPath[MAX_PATH];

         dwSize = GetModuleFileName(m_hDll, szDllPath, MAX_PATH);

         if ( dwSize > 0 ) {
            m_szFullPath = new TCHAR[dwSize + 1];
            _tcscpy(m_szFullPath, szDllPath);
         }
      }
   }

   return m_szFullPath;
} // LoadProcWrapperBase::GetModuleLoadPath()

const bool LoadProcWrapperBase::SetThrowExceptionOnMissingEntry(IN const bool _bThrowOnMissing)
{
   bool  bOldValue = m_bThrowOnMissing;

   m_bThrowOnMissing = _bThrowOnMissing;

   return bOldValue;
} // LoadProcWrapperBase::SetThrowExceptionOnMissingEntry

void LoadProcWrapperBase::SetNames(const TCHAR * const _szDllName, const char  * const _szExportName)
{
   if ( _szDllName != NULL ) {
      m_szDllName = new TCHAR[lstrlen(_szDllName) + 1];
      strcpy(m_szDllName, _szDllName);
   }
   if ( _szExportName != NULL ) {
      m_szExportName = new char[strlen(_szExportName) + 1];
      strcpy(m_szExportName, _szExportName);
   }
} // LoadProcWrapperBase::SetNames()

void LoadProcWrapperBase::GetTheFunction(void)
{
   LoadTheDll();
   if ( m_hDll == NULL ) {
      throw (1);
   }

   if ( m_pf == NULL && m_szExportName != NULL ) {
      m_pf = GetProcAddress(m_hDll, m_szExportName);
   }

   if ( m_pf == NULL ) {
      if ( m_bThrowOnMissing ) {
         throw (2);
      } else {
         // pass
      }
   }
} // LoadProcWrapperBase::GetTheFunction()

void LoadProcWrapperBase::LoadTheDll(void)
{
   // Not optimal, but who cares?
   for ( int idx = 0; idx < sizeof(m_szDllPath) / sizeof(m_szDllPath[0]); idx++ ) {
      LoadTheDllFrom(m_szDllPath[idx]);
   }
   LoadTheDllFrom(NULL);
} // LoadProcWrapperBase::LoadTheDll()

void LoadProcWrapperBase::LoadTheDllFrom(IN const TCHAR * const szPath)
{
   TCHAR    szDllPath[MAX_PATH] = _T("\0");

   if ( m_hDll != NULL ) {
      return;
   }

   if ( lstrlen(m_szDllName) + 1 > MAX_PATH ) {
      throw(71);
   }

   if ( szPath != NULL ) {

      if ( lstrlen(szPath) + lstrlen(m_szDllName) + 3 > MAX_PATH ) {
         throw(72);
      }

      if ( lstrlen(szPath) > 0 ) {
         _tcscpy(szDllPath, szPath);
         _tcscat(szDllPath, _T("\\"));
      }
   }

   _tcscat(szDllPath, m_szDllName);

   m_hDll = LoadLibrary(szDllPath);
} // LoadProcWrapperBase::LoadTheDllFrom()

void LoadProcWrapperBase::ResetPaths(void)
{
   for ( int idx = 0; idx < sizeof(m_szDllPath) / sizeof(m_szDllPath[0]); idx++ ) {
      delete[] m_szDllPath[idx];
      m_szDllPath[idx] = NULL;
   }
   delete[] m_szFullPath;
   m_szFullPath = NULL;
} // LoadProcWrapperBase::ResetPaths()

void LoadProcWrapperBase::SetPreferredPathI(IN const TCHAR * const _p, IN const int n)
{
   TCHAR    szDllPath[MAX_PATH];
   CLsd     lsd(_p);

   if ( n < 0 || n >  sizeof(m_szDllPath) / sizeof(m_szDllPath[0]) ) {
      throw 61;
   }

   if ( lsd.GetPath(szDllPath, MAX_PATH) > 0 ) {
      m_szDllPath[n] = new TCHAR[lstrlen(szDllPath) + 1];
      strcpy(m_szDllPath[n], szDllPath);
   } else {
      if ( _p == NULL || lstrlen(_p) == 0 ) {
         throw(81);
      }

      m_szDllPath[n] = new TCHAR[lstrlen(_p) + 1];
      strcpy(m_szDllPath[n], _p);
   }
} // LoadProcWrapperBase::SetPreferredPathI()


//////////////////////////////////////////////////////////////////////
// Load System Directory - a helper class
// Rather not optimal but gives us what we need
//////////////////////////////////////////////////////////////////////

CLsd::CLsd(IN const eLOADSPECIALDIRECTORY _eT)
 : m_t(_eT)
{
} // (1) CLsd::CLsd() 

CLsd::CLsd(IN const TCHAR * _s)
{
   switch ( (int)_s ) {
      case E_WINDOWSDIR:
         m_t = E_WINDOWSDIR;
         break;
      case E_SYSTEMDIR:
         m_t = E_SYSTEMDIR;
         break;
      case E_CURRENTDIR:
         m_t = E_CURRENTDIR;
         break;
      case E_EXEDIR:
         m_t = E_EXEDIR;
         break;
      default:
         m_t = E_UNKNOWN;
         break;
   }
} // (2) CLsd::CLsd() 

CLsd::operator const TCHAR *const ()
{
   return reinterpret_cast<TCHAR *>(m_t);
}

UINT CLsd::GetPath(OUT TCHAR * szOutBuf, IN DWORD dwOutBufSize)
{
   UINT  ret = 0;

   switch ( m_t ) {
      case E_WINDOWSDIR:
         ret = GetWindowsDirectory(szOutBuf, dwOutBufSize);
         break;
      case E_SYSTEMDIR:
         ret = GetSystemDirectory(szOutBuf, dwOutBufSize);
         break;
      case E_CURRENTDIR:
         ret = GetCurrentDirectory(dwOutBufSize, szOutBuf);
         break;
      case E_EXEDIR:
         ret = GetModuleFileName(NULL, szOutBuf, dwOutBufSize);
         break;
      default:
         break;
   }

   return ret;
} // CLsd::GetPath()
