// LoadProcWrapperBase.h: interface for the LoadProcWrapperBase class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOADPROCWRAPPERBASE_H__1B5F3CBF_A737_4AA2_9E4D_62A85E3122B9__INCLUDED_)
#define AFX_LOADPROCWRAPPERBASE_H__1B5F3CBF_A737_4AA2_9E4D_62A85E3122B9__INCLUDED_

#include <windows.h>
#include <tchar.h>


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// Load System Directory - a helper class
class CLsd
{
public:
   typedef  enum _eLoadSpecialDirectory {E_UNKNOWN = 1, E_WINDOWSDIR, E_SYSTEMDIR, E_CURRENTDIR, E_EXEDIR} eLOADSPECIALDIRECTORY;

private:
   eLOADSPECIALDIRECTORY   m_t;

public:
   CLsd(IN const eLOADSPECIALDIRECTORY _eT);
   CLsd(IN const TCHAR * _s);
   operator const TCHAR *const ();
   UINT GetPath(OUT TCHAR * szOutBuf, IN DWORD dwOutBufSize);
};

// Root of the hierarchy - all base functionality. 
// Notice all functionality not relying on templates is located here
class LoadProcWrapperBase
{
protected:
   TCHAR *  m_szDllName;
   TCHAR *  m_szDllPath[3];
   TCHAR *  m_szFullPath;
   char  *  m_szExportName;
   HMODULE  m_hDll;
   void  *  m_pf;
   bool     m_bThrowOnMissing;

public:
   LoadProcWrapperBase();
   virtual ~LoadProcWrapperBase();

   virtual void Reset(void);
   virtual void SetPreferredPath(IN const TCHAR * const _p1);
   virtual void SetPreferredPath(IN const TCHAR * const _p1, IN const TCHAR * const _p2);
   virtual void SetPreferredPath(IN const TCHAR * const _p1, IN const TCHAR * const _p2, IN const TCHAR * const _p3);
   virtual const TCHAR * const GetModuleLoadPath(void);
   virtual const bool SetThrowExceptionOnMissingEntry(IN const bool _bThrowOnMissing);

protected:
   virtual void SetNames(const TCHAR * const _szDllName, const char  * const _szExportName);
   virtual void GetTheFunction(void);
   virtual void LoadTheDll(void);
   virtual void LoadTheDllFrom(IN const TCHAR * const szPath);

private:
   void ResetPaths(void);
   void SetPreferredPathI(IN const TCHAR * const _p, IN const int n);
};

// Common for all templates. Used to store the template parameters
// which define the DLL name and the function name
// Also defines the default return value - if no exception is to be thrown
template
<
   const TCHAR * const * _pszDllName = NULL, 
   const char  * const * _pszExportName = NULL
> 
class LoadProcWrapperLower : public LoadProcWrapperBase
{
public:
   LoadProcWrapperLower() : LoadProcWrapperBase()
   { 
      if ( _pszDllName != NULL && _pszExportName != NULL  ) {
         SetNames(*_pszDllName, *_pszExportName);
      }
   }
};

// Upgrades with the option to return a default value
// typename T_RET must define a meaningful assignment operator
template
<
   typename T_RET, 
   const TCHAR * const * _pszDllName = NULL, 
   const char  * const * _pszExportName = NULL
> 
class LoadProcWrapperRetVal : public LoadProcWrapperLower<_pszDllName, _pszExportName>
{
protected:
   T_RET    m_DefaultRet;

public:
   LoadProcWrapperRetVal() : LoadProcWrapperLower<_pszDllName, _pszExportName>(), m_DefaultRet(0) {}
   
protected:
   virtual T_RET  SetDefaultValue(T_RET const & _DefaultRet)
   {
      T_RET    oldRet = m_DefaultRet;

      m_DefaultRet = _DefaultRet;
      
      return _DefaultRet;
   }

   virtual T_RET & GetDefaultValue(void)
   {
      return (m_DefaultRet);
   }
};


#endif // !defined(AFX_LOADPROCWRAPPERBASE_H__1B5F3CBF_A737_4AA2_9E4D_62A85E3122B9__INCLUDED_)
